package game.obj.system
{
	import flash.display.BitmapData;
	import flash.geom.Rectangle;
	import flash.geom.Point;
	import game.obj.GameObject;
	
	public class BGLayer extends GameObject
	{
		private var depth:String;
		
		private var chip:BitmapData;
		private var table:Array;
		
		private const CHIP_SIZE:int = 24;
		
		public function BGLayer(depth:String, width:int, height:int)
		{
			table = new Array();
			for(var y:int = 0; y < height; y++) {
				table[y] = new Array();
				for(var x:int = 0; x < width; x++) {
					table[y][x] = 0;
				}
			}
			this.depth = depth;
		}
		
		override public function init():void
		{
			registerObject("PRIO_SYSTEM", depth);
		}
		
		public function setGraphic(chip:BitmapData):void
		{
			this.chip = chip;
		}
		
		public function setTable(table:Array):void
		{
			this.table = table;
		}
		
		public function putChip(num:int, x:int, y:int):void
		{
			if(y < 0 || y >= table.length) {return;}
			if(x < 0 || x >= table[y].length) {return;}
			
			table[y][x] = num;
		}
		
		override public function draw(surface:BitmapData):void
		{
			var x1:int = scene.getCameraX()/24;
			var y1:int = scene.getCameraY()/24;
			var x2:int = x1+15;
			var y2:int = y1+11;
			
			for(var y:int = y1; y < y2; y++)
			{
				if(y < 0 || y >= table.length) {continue;}
				for(var x:int = x1; x < x2; x++)
				{
					if(x < 0 || x >= table[y].length) {continue;}
					if(table[y][x] == 0) {continue;}
					
					var chip_x:int = table[y][x] % 8;
					var chip_y:int = table[y][x] / 8;
					
					surface.copyPixels(
						chip,
						new Rectangle(chip_x*CHIP_SIZE, chip_y*CHIP_SIZE, CHIP_SIZE, CHIP_SIZE),
						new Point(x*CHIP_SIZE-scene.getCameraX(), y*CHIP_SIZE-scene.getCameraY())
					);
				}
			}
			return;
		}
	}
}