package game.obj.system
{
	import game.obj.GameObject;
	import flash.display.BitmapData;
	import flash.geom.Rectangle;
	import game.obj.box.*;
	import flash.geom.Point;
	import game.system.GameData;
	import common.List;
	
	public class HUD extends GameObject
	{
		private var gamedata:GameData;
		private var bmp:BitmapData = new BitmapData(256, 256, true, 0);
		
		public function HUD(gamedata:GameData)
		{
			this.gamedata = gamedata;
		}
		
		override public function init():void
		{
			registerObject("PRIO_SYSTEM", "DEPTH_MESSAGE");
		}
		
		override public function draw(surface:BitmapData):void
		{
			var txt_stage:String = gamedata.course+"-"+gamedata.stage;
			var txt_score:String = MathEx.addZero(gamedata.score, 6);
			var txt_time:String = gamedata.timeToString(gamedata.time);
			
			func_text.draw("16x8_orange", surface, "STAGE", 8, 8);
			func_text.draw("16x8_blue", surface, txt_stage, 8+48, 8);
			func_text.draw("16x8_orange", surface, "TIME", 8, 240-8-16);
			func_text.draw("16x8_blue", surface, txt_time, 8+40, 240-8-16);
			
			if(gamedata.mode == "SEQUENTIAL")
			{
				func_text.draw("16x8_orange", surface, "SCORE", 320-8-8*6-8*6, 8);
				func_text.draw("16x8_blue", surface, txt_score, 320-8-8*6, 8);
			}
			
			//マップ
			var width:int = stage.table[0].length;
			var height:int = stage.table.length;
			
			bmp.fillRect(new Rectangle(2, 2, width*2 + 4, height*2 + 4), 0x80000000);
			bmp.fillRect(new Rectangle(0, 0, width*2 + 4, height*2 + 4), 0xFFFFFFFF);
			bmp.fillRect(new Rectangle(1, 1, width*2 + 2, height*2 + 2), 0x80000000);
			
			var color:uint;
			
			for(var y:int = 0; y < height; y++)
			{
				for(var x:int = 0; x < width; x++)
				{
					switch(stage.table[y][x])
					{
					case 0:
						continue;
					default:
						color = 0xC0FFFFFF;
						break;
					case 2:
						bmp.fillRect(new Rectangle(2+x*2, 2+y*2, 2, 1), 0xE8FFE0A0);
						continue;
					case 3:
					case 4:
						bmp.fillRect(new Rectangle(2+x*2, 2+y*2, 2, 1), 0xE840E0FF);
						bmp.fillRect(new Rectangle(2+x*2, 2+y*2+1, 2, 1), 0xC0FFFFFF);
						continue;
					}
					bmp.fillRect(new Rectangle(2+x*2, 2+y*2, 2, 2), color);
				}
			}
			
			var objlist:List;
			
			objlist = scene.func_obj.searchPriority("PRIO_CHECKPOINT");
			for each(var p:* in objlist)
			{
				bmp.fillRect(new Rectangle(2+p.x/12-1, 2+p.y/12-1, 2, 2), 0xC00080FF)
			}
			
			objlist = scene.func_obj.searchPriority("PRIO_BOX");
			for each(var box:* in objlist)
			{
				if(box is Box)  {color = 0xFFFF8000;}
				if(box is Cage) {color = 0xFFC0A0B0;}
				bmp.fillRect(new Rectangle(2+box.x/12-1, 2+box.y/12-1, 2, 2), color)
			}
			
			objlist = scene.func_obj.searchPriority("PRIO_PLAYER");
			if(objlist.length != 0)
			{
				bmp.fillRect(new Rectangle(2+objlist[0].x/12-1, 2+objlist[0].y/12-2, 2, 2), 0xFF80FF00);
			}
			
			surface.copyPixels(bmp, new Rectangle(0, 0, width*2+6, height*2+6), new Point(320-8 - width*2, 240-8 - height*2));
			return;
		}
	}
}