package game.obj.system
{
	import game.obj.GameObject;
	import flash.display.BitmapData;
	
	public class ListMenu extends GameObject
	{
		public var itemlist:Array = new Array();
		
		public var width:int = 160;
		public var linespace:int = 8;
		
		public var length:int = 0;
		public var selectedindex:int = 0;
		public var currentitem:MenuItem;
		
		public var enabled:Boolean = true;
		
		private var decided:Boolean = false;
		
		override public function init():void
		{
			registerObject("PRIO_SYSTEM", "DEPTH_MESSAGE");
			
			decided = false;
			
			for each(var obj:* in itemlist)
			{
				createObject(obj);
				obj.width = width;
			}
			update();
			
			changeAction("act_select");
		}
		
		override public function erase():void
		{
			for each(var obj:* in itemlist)
			{
				obj.vanish();
			}
		}
		
		public function act_select():void
		{
			if(enabled == true && currentitem != null)
			{
				if(func_key.isPress("BTN_A") && currentitem.button == true)
				{
					playSE("SE_ENTER");
					
					decided = true;
					changeAction("act_decide");
					return;
				}
				
				if(currentitem.data != null)
				{
					if(func_key.isPress("LEFT", true) && currentitem.index > 0)
					{
						playSE("SE_SELECT");
						currentitem.index--;
					}
					else if(func_key.isPress("RIGHT", true) && currentitem.index < currentitem.data.length-1)
					{
						playSE("SE_SELECT");
						currentitem.index++;
					}
				}
				
				if(func_key.isPress("DOWN", true))
				{
					playSE("SE_SELECT");
					selectedindex++;
					
					if(selectedindex == itemlist.length) {
						selectedindex = 0;
					}
				}
				else if(func_key.isPress("UP", true))
				{
					playSE("SE_SELECT");
					selectedindex--;
					
					if(selectedindex == -1) {
						selectedindex = itemlist.length-1;
					}
				}
			}
			update();
			
			currentitem = itemlist[selectedindex];
			return;
		}
		
		public function act_decide():void
		{
			if(count <= 12)
			{
				for each(var item:MenuItem in itemlist)
				{
					if(item.selected == true)
					{
						if(count % 6 < 3) {
							item.visible = false;
						}
						else {
							item.visible = true;
						}
					}
					else {
						item.visible = false;
					}
				}
			}
			else {
				vanish();
			}
			
			return;
		}
		
		// --------------------------------//
		// レイアウト
		// --------------------------------//
		public function update():void
		{
			var y:Number = this.y;
			
			for(var i:int = 0; i < itemlist.length; i++)
			{
				itemlist[i].x = x;
				itemlist[i].y = y;
				y += 16 + linespace;
				
				if(i == selectedindex) {
					itemlist[i].selected = true;
				}
				else {
					itemlist[i].selected = false;
				}
			}
		}
		
		// --------------------------------//
		// 外向けの関数
		// --------------------------------//
		public function addItem(label:String, button:Boolean = true, data:Array = null):void
		{
			itemlist.push(new MenuItem(label, button, data));
			length++;
			return;
		}
		
		public function getItem(name:String):MenuItem
		{
			for each(var item:MenuItem in itemlist)
			{
				if(item.label == name)
				{
					return item;
				}
			}
			return null;
		}
		
		public function isDecided():Boolean
		{
			return decided;
		}
	}
}
