package game.scene
{
	import game.system.SceneBase;
	import game.obj.player.Player;
	import game.scene.maingame.MainStage;
	import game.obj.system.HUD;
	import game.system.GameData;
	import game.obj.system.CheckPoint;
	import game.obj.system.SystemMessage;
	import game.obj.system.Message;
	import game.obj.system.Picture;
	import flash.display.BitmapData;
	import game.obj.system.ListMenu;
	import game.system.PlayData;
	import game.system.Database;
	import common.List;
	import game.obj.system.FadeIn;
	
	public class MainGameScene extends SceneBase
	{
		private var gamedata:GameData;
		private var cleartime:Number = 0;
		
		public var player:Player;
		private var stage:MainStage;
		private var hud:HUD;
		
		public function MainGameScene(gamedata:GameData):void
		{
			this.gamedata = gamedata;
		}
		
		override public function init():void
		{
			player = createObject(new Player());
			
			/*
			gamedata.course = 2;
			gamedata.stage = 8;
			//*/
			
			stage = new MainStage();
			stage.scene = this;
			stage.create(gamedata.course, gamedata.stage);
			
			screen.setColor(0, 0, 0, 1, 0, 0, 0, 0);
			switch(gamedata.mode)
			{
			case "SEQUENTIAL":
				changeAction("act_standby");
				break;
			case "SINGLE":
				gamedata.time = 0;
				changeAction("act_standby");
				break;
			case "STAGESELECT":
				changeAction("act_stageselect");
				break;
			}
		}
		
		// --------------------------------//
		// メインゲーム
		// --------------------------------//
		public function act_standby():void
		{
			if(count == 0)
			{
				camera_homepos();
				
				hud = createObject(new HUD(gamedata));
				hud.scene = this;
				hud.stage = stage;
				
				if(gamedata.mode == "SEQUENTIAL" && gamedata.stage == 1 && gamedata.time == 90)
				{
					player.visible = false;
					changeAction("act_startcourse");
				}
				else
				{
					player.func_key.unlock();
					changeAction("act_main");
				}
			}
		}
		
		private var course:Message = new Message("16x8_orange");
		private var ready:Message = new Message("16x8_blue");
		
		public function act_startcourse():void
		{
			if(count == 0) {
				createObject(new FadeIn(15));
			}
			
			if(count == 20)
			{
				course.str = "COURSE "+gamedata.course;
				course.x = 160-course.str.length*4;
				course.y = 100;
				createObject(course);
				
				player.changeAction("act_toujo");
			}
			
			if(count == 60)
			{
				ready.str = "READY";
				ready.x = 160-ready.str.length*4;
				ready.y = 124;
				createObject(ready);
			}
			
			if(count == 105)
			{
				course.vanish();
				ready.vanish();
			}
			
			if(count == 110)
			{
				changeAction("act_main");
			}
		}
		
		public function act_main():void
		{
			if(count == 0)
			{
				screen.setColor(1, 1, 1, 1, 0, 0, 0, 0);
				func_bgm.play("BGM_COURSE"+gamedata.course);
				func_bgm.setVolume(100);
				player.func_key.unlock();
			}
			
			if(func_key.isPress("ENTER"))
			{
				playSE("SE_PAUSE");
				changeAction("act_pause");
				return;
			}
			
			stage.chip_animation();
			camera_main();
			
			//クリア判定
			var objlist:List = func_obj.searchPriority("PRIO_CHECKPOINT");
			var entercount:int = 0;
			for each(var checkpoint:CheckPoint in objlist)
			{
				if(checkpoint.enter == true) {
					entercount++;
				}
			}
			if(entercount == objlist.length)
			{
				gamedata.score += (gamedata.time*gamedata.time) * gamedata.stage;
				gamedata.records.push(cleartime);
				
				if(gamedata.mode == "SEQUENTIAL" && stage.isFinal(gamedata.course, gamedata.stage))
				{
					changeAction("act_allclear");
				}
				else {
					changeAction("act_stageclear");
				}
				return;
			}
			
			if(gamedata.mode == "SEQUENTIAL")
			{
				gamedata.time -= 1/45;
				cleartime += 1/45;
				
				//時間切れ判定
				if(gamedata.time <= 0)
				{
					gamedata.time = 0;
					changeAction("act_timeout");
					return;
				}
			}
			else
			{
				gamedata.time += 1/45;
			}
			
			return;
		}
		
		public function act_timeout():void
		{
			if(count == 0)
			{
				func_obj.lock();
				func_bgm.fadeout();
				
				playSE("SE_TIMEOUT");
				createObject(new SystemMessage("TIMEOUT"));
			}
			
			if(count >= 20) {
				if(func_key.isPress("BTN_A")) {
					changeAction_fadeout("act_gotoresult");
				}
			}
		}
		
		public function act_stageclear():void
		{
			if(count == 0)
			{
				func_obj.lock();
				
				playSE("SE_STAGECLEAR");
				createObject(new SystemMessage("STAGECLEAR"));
			}
			
			if(gamedata.mode == "SEQUENTIAL")
			{
				if(count == 20)
				{
					var obj:* = createObject(new Message("8px", "TIME EXTEND +15SEC"));
					obj.x = 160-obj.str.length*4;
					obj.y = 140;
					
					gamedata.time += 15;
				}
				
				if(count >= 20) {
					if(func_key.isPress("BTN_A")) {
						changeAction_fadeout("act_gotonextstage");
					}
				}
			}
			else {
				if(count == 20)
				{
					if(PlayData.records.isBest(gamedata.course, gamedata.stage, gamedata.time))
					{
						var obj:* = createObject(new Message("8px", "NEW RECORD!"));
						obj.x = 160-obj.str.length*4;
						obj.y = 140;
						
						PlayData.records.entry(gamedata.course, gamedata.stage, gamedata.time);
						PlayData.records.saveData();
					}
				}
				
				if(count >= 20) {
					if(func_key.isPress("BTN_A")) {
						func_bgm.fadeout();
						changeAction_fadeout("act_gotomainmenu");
					}
				}
			}
		}
		
		public function act_allclear():void
		{
			if(count == 0)
			{
				func_obj.lock();
				func_bgm.fadeout();
				
				playSE("SE_ALLCLEAR");
				createObject(new SystemMessage("ALLCLEAR"));
			}
			
			if(count >= 30) {
				if(func_key.isPress("BTN_A")) {
					changeAction_fadeout("act_gotoresult");
				}
			}
		}
		
		// --------------------------------//
		// ポーズメニュー
		// --------------------------------//
		private var pause_back:Picture;
		private var arrow:Picture;
		private var pausemenu:ListMenu;
		
		public function act_pause():void
		{
			if(count == 0)
			{
				func_bgm.setVolume(50);
				func_obj.lock();
				
				//メニュー作成
				pausemenu = new ListMenu();
				pausemenu.addItem("CONTINUE");
				pausemenu.addItem("MAP");
				pausemenu.addItem("RETRY");
				
				if(gamedata.mode == "SEQUENTIAL") {pausemenu.addItem("ABANDON");}
				
				pausemenu.addItem("BACK TO MENU");
				pausemenu.addItem("CONTINUE");
				
				pausemenu.x = 104;
				pausemenu.y = 136 - (pausemenu.length*10-2);
				pausemenu.linespace = 4;
				
				//メニュー背景
				pause_back = createObject(new Picture(new BitmapData(144, 20*pausemenu.length+16-4, true, 0xC0000000), "DEPTH_MESSAGE"));
				pause_back.x = 160 - pause_back.bmp.width/2;
				pause_back.y = 136 - pause_back.bmp.height/2;
				
				createObject(pausemenu);
			}
			
			if(pausemenu.isDecided())
			{
				changeAction("act_pause_decide");
				return;
			}
			else if(func_key.isPress("BTN_B") || func_key.isPress("ENTER"))
			{
				playSE("SE_CANCEL");
				func_obj.unlock();
				
				pausemenu.vanish();
				pause_back.vanish();
				changeAction("act_main");
				return;
			}
		}
		
		public function act_pause_decide():void
		{
			if(count == 15)
			{
				pause_back.vanish();
				
				switch(pausemenu.currentitem.label)
				{
				case "MAP":
					changeAction("act_viewmap");
					break;
				case "RETRY":
					changeAction_fadeout("act_rebootmaingame");
					break;
				case "ABANDON":
					func_bgm.fadeout();
					
					hud.visible = false;
					gamedata.stage = 1;
					gamedata.score = 0;
					gamedata.time = 90;
					gamedata.records = new Array();
					
					changeAction_fadeout("act_rebootmaingame");
					break;
				case "BACK TO MENU":
					func_bgm.fadeout();
					changeAction_fadeout("act_gotomainmenu");
					break;
				case "CONTINUE":
					func_obj.unlock();
					changeAction("act_main");
					break;
				}
				return;
			}
		}
		
		public function act_viewmap():void
		{
			if(count == 0)
			{
				hud.visible = false;
				arrow = createObject(new Picture(Database.getItem("BMP_ARROW")));
			}
			if(count % 30 < 10) {
				arrow.visible = false;
			}
			else {
				arrow.visible = true;
			}
			
			camera_control();
			
			if(func_key.isPress("BTN_B") || func_key.isPress("ENTER"))
			{
				playSE("SE_CANCEL");
				
				hud.visible = true;
				camera_playerpos();
				arrow.vanish();
				
				changeAction("act_pause");
			}
		}
		
		// --------------------------------//
		// ステージセレクト
		// --------------------------------//
		private var txt_stage:Message;
		private var txt_besttime:Message;
		
		public function act_stageselect():void
		{
			stage.chip_animation();
			
			if(count == 0)
			{
				func_bgm.play("BGM_STANDBY");
				camera_homepos();
				
				var bmp:Picture = createObject(new Picture(new BitmapData(320, 240, true, 0x80000000), "DEPTH_MESSAGE"));
				
				var mes:Message = createObject(new Message("8px", "CURSOR-SELECT  Z-DECIDE  SHIFT-RETURN"));
				mes.x = 160 - mes.str.length*4;
				mes.y = 8;
				
				txt_stage = createObject(new Message("16x8_orange", "STAGE "+gamedata.course+"-"+gamedata.stage));
				txt_besttime = createObject(new Message("16x8_blue", "BEST TIME "+gamedata.timeToString(PlayData.records.data.record[gamedata.course][gamedata.stage])));
				
				txt_stage.x = 160 - txt_stage.str.length*4;
				txt_stage.y = 100;
				txt_besttime.x = 160 - txt_besttime.str.length*4;
				txt_besttime.y = 120;
			}
			
			if(count < 4) {
				return;
			}
			else {
				screen.setColor(1, 1, 1, 1, 0, 0, 0, 0);
			}
			
			if(func_key.isPress("BTN_A"))
			{
				playSE("SE_ENTER");
				gamedata.mode = "SINGLE";
				changeAction("act_stagedecide");
				return;
			}
			
			if(func_key.isPress("LEFT", true))
			{
				playSE("SE_SELECT");
				gamedata.stage--;
				changeAction("act_changestage");
				return;
			}
			if(func_key.isPress("RIGHT", true))
			{
				playSE("SE_SELECT");
				gamedata.stage++;
				changeAction("act_changestage");
				return;
			}
			
			if(func_key.isPress("BTN_B"))
			{
				func_bgm.fadeout();
				playSE("SE_CANCEL");
				changeAction_fadeout("act_gotomainmenu");
				return;
			}
		}
		
		public function act_changestage():void
		{
			if(gamedata.course == 1 && gamedata.stage == 0)
			{
				gamedata.course = 2;
				gamedata.stage = 8;
			}
			if(gamedata.course == 1 && gamedata.stage == 7)
			{
				gamedata.course = 2;
				gamedata.stage = 1;
			}
			if(gamedata.course == 2 && gamedata.stage == 0)
			{
				gamedata.course = 1;
				gamedata.stage = 6;
			}
			if(gamedata.course == 2 && gamedata.stage == 9)
			{
				gamedata.course = 1;
				gamedata.stage = 1;
			}
			
			changeScene(new MainGameScene(gamedata));
		}
		
		public function act_stagedecide():void
		{
			stage.chip_animation();
			
			if(count == 0) {
				txt_besttime.vanish();
			}
			
			if(count % 6 < 3) {
				txt_stage.visible = false;
			}
			else {
				txt_stage.visible = true;
			}
			
			if(count == 12)
			{
				txt_stage.vanish();
				changeAction_fadeout("act_rebootmaingame");
			}
		}
		
		// --------------------------------//
		// シーン変更
		// --------------------------------//
		public function act_gotonextstage():void
		{
			gamedata.stage++;
			changeScene(new MainGameScene(gamedata));
			return;
		}
		
		public function act_gotomainmenu():void
		{
			changeScene(new MenuScene());
			return;
		}
		
		public function act_gotoresult():void
		{
			changeScene(new ResultScene(gamedata));
			return;
		}
		
		public function act_rebootmaingame():void
		{
			changeScene(new MainGameScene(gamedata));
		}
		
		// --------------------------------//
		// カメラ
		// --------------------------------//
		private var camera_xpan:Number = 0;
		private var camera_ypan:Number = 0;
		private var camera_zure:Number = 0;
		private var camera_zure2:Number = 0;
		
		public function camera_main():void
		{
			camera_hosei();
			
			camera_x = (camera_x + camera_xpan) / 2;
			camera_y = (camera_y*3 + camera_ypan) / 4;
			camera_xpan = player.x-160+camera_zure;
			camera_ypan = player.y-150;
			
			if(player.dir == "LEFT") {camera_zure2 = (camera_zure2*6 - 48) / 7;}
			if(player.dir == "RIGHT") {camera_zure2 = (camera_zure2*6 + 48) / 7;}
			camera_zure = (camera_zure*9 + camera_zure2) / 10;
		}
		
		public function camera_control():void
		{
			if(func_key.isDown("LEFT"))  {camera_x -= 8;}
			if(func_key.isDown("RIGHT")) {camera_x += 8;}
			if(func_key.isDown("UP"))    {camera_y -= 8;}
			if(func_key.isDown("DOWN"))  {camera_y += 8;}
			
			camera_hosei();
		}
		
		public function camera_playerpos():void
		{
			camera_x = player.x-160+camera_zure;
			camera_y = player.y-150;
			
			camera_hosei();
		}
		
		public function camera_homepos():void
		{
			camera_x = player.x-160;
			camera_y = player.y-160;
			camera_xpan = player.x-150;
			camera_ypan = player.y-150;
			
			camera_hosei();
		}
		
		public function camera_hosei():void
		{
			if(camera_x < 0) {camera_x = 0;}
			if(camera_y < 0) {camera_y = 0;}
			if(camera_x > stage.width*24 - 320) {camera_x = stage.width*24 - 320;}
			if(camera_y > stage.height*24 - 240) {camera_y = stage.height*24 - 240;}
			
			if(camera_xpan < 0) {camera_xpan = 0;}
			if(camera_ypan < 0) {camera_ypan = 0;}
			if(camera_xpan > stage.width*24 - 320) {camera_xpan = stage.width*24 - 320;}
			if(camera_ypan > stage.height*24 - 240) {camera_ypan = stage.height*24 - 240;}
		}
	}
}