﻿package game.scene
{
	import game.system.SceneBase;
	import game.system.GameData;
	import game.obj.system.ListMenu;
	import game.obj.system.Message;
	import common.AppInfo;
	import game.system.PlayData;
	import game.obj.system.FadeIn;
	
	public class MenuScene extends SceneBase
	{
		private var mainmenu:ListMenu = new ListMenu();
		private var option:ListMenu = new ListMenu();
		
		private var selectedcourse:int;
		private var course:Object = new Object();
		
		private var gamedata:GameData = new GameData();
		
		override public function init():void
		{
			func_bgm.play("BGM_MENU");
			createObject(new FadeIn(45));
			
			mainmenu.addItem("START");
			mainmenu.addItem("STAGE SELECT");
			mainmenu.addItem("RANKING");
			mainmenu.addItem("OPTION");
			mainmenu.linespace = 12;
			
			var bgmlist:Array = new Array
			(
				"STOP",
				"COURSE1",
				"COURSE2",
				"ENTRY",
				"MENU",
				"STANDBY"
			);
			var selist:Array = new Array
			(
				"STOP",
				"JUMP",
				"CHAKUCHI",
				"GRAB",
				"THROW",
				"BOXCHAKUCHI",
				"SUBERI",
				"STAGECLEAR",
				"ALLCLEAR",
				"TIMEOUT",
				"SELECT",
				"ENTER",
				"CANCEL",
				"PAUSE",
				"START"
			);
			
			option.addItem("FPS", false, new Array("OFF", "ON"));
			option.addItem("BGM TEST", false, bgmlist);
			option.addItem("SE TEST", false, selist);
			option.linespace = 12;
			
			createObject(new BG());
			
			var mes:Message = createObject(new Message("8px", "CURSOR-SELECT  Z-DECIDE  SHIFT-RETURN"));
			mes.x = 160 - mes.str.length*4;
			mes.y = 8;
			
			changeAction("act_mainmenu");
		}
		
		// --------------------------------//
		// メインメニュー
		// --------------------------------//
		public function act_mainmenu():void
		{
			if(count == 0)
			{
				mainmenu.x = 88;
				mainmenu.y = 112;
				createObject(mainmenu);
			}
			
			if(mainmenu.isDecided())
			{
				changeAction("act_mainmenu_decide");
			}
		}
		
		public function act_mainmenu_decide():void
		{
			if(count == 15)
			{
				switch(mainmenu.currentitem.label)
				{
				case "START":
					changeAction("act_courseselect");
					break;
				case "STAGE SELECT":
					changeAction_fadeout("act_gotostageselect");
					break;
				case "RANKING":
					changeAction_fadeout("act_gotoranking");
					break;
				case "OPTION":
					changeAction("act_option");
					break;
				}
				return;
			}
		}
		
		// --------------------------------//
		// コースセレクト
		// --------------------------------//
		public function act_courseselect():void
		{
			if(count == 0)
			{
				selectedcourse = 1;
				
				course[1] = createObject(new Course(1));
				course[2] = createObject(new Course(2));
			}
			
			if(func_key.isPress("BTN_B"))
			{
				playSE("SE_CANCEL");
				course[1].vanish();
				course[2].vanish();
				changeAction("act_mainmenu");
				return;
			}
			
			if(func_key.isPress("BTN_A"))
			{
				playSE("SE_ENTER");
				gamedata.course = selectedcourse;
				changeAction("act_coursedecide");
				return;
			}
			
			if(func_key.isPress("UP", true) || func_key.isPress("DOWN", true))
			{
				playSE("SE_SELECT");
				
				switch(selectedcourse)
				{
				case 1: selectedcourse = 2; break;
				case 2: selectedcourse = 1; break;
				}
			}
			
			if(selectedcourse == 1)
			{
				course[1].selected = true;
				course[2].selected = false;
			}
			if(selectedcourse == 2)
			{
				course[2].selected = true;
				course[1].selected = false;
			}
		}
		
		public function act_coursedecide():void
		{
			if(count == 0) {
				func_bgm.fadeout();
				course[selectedcourse].changeAction("act_decide");
			}
			
			if(count == 18)
			{
				course[1].vanish();
				course[2].vanish();
				
				changeAction_fadeout("act_gotomaingame");
			}
		}
		
		// --------------------------------//
		// コンフィグ
		// --------------------------------//
		public function act_option():void
		{
			if(count == 0)
			{
				option.x = 80;
				option.y = 128;
				option.width = 160;
				option.selectedindex = 0;
				
				option.getItem("FPS").currentdata = PlayData.option.data.fps;
				
				createObject(option);
			}
			
			if(option.currentitem != null)
			{
				var data:String = option.currentitem.currentdata;
				
				switch(option.currentitem.label)
				{
				case "FPS":
					if(data == "OFF") {AppInfo.hideFPS();}
					if(data == "ON") {AppInfo.viewFPS();}
					break;
				case "BGM TEST":
					if(func_key.isPress("BTN_A"))
					{
						func_bgm.stop();
						if(data != "STOP") {
							func_bgm.play("BGM_"+data);
						}
					}
					break;
				case "SE TEST":
					if(func_key.isPress("BTN_A"))
					{
						if(data == "STOP") {stopSE();}
						else {playSE("SE_"+data);}
					}
					break;
				}
			}
			
			if(func_key.isPress("BTN_B"))
			{
				PlayData.option.data.fps = option.getItem("FPS").currentdata;
				PlayData.option.saveData();
				
				func_bgm.play("BGM_MENU");
				
				playSE("SE_CANCEL");
				option.vanish();
				changeAction("act_mainmenu");
				return;
			}
		}
		
		// --------------------------------//
		// シーン変更
		// --------------------------------//
		public function act_gotomaingame():void
		{
			gamedata.mode = "SEQUENTIAL";
			changeScene(new MainGameScene(gamedata));
			return;
		}
		
		public function act_gotostageselect():void
		{
			gamedata.mode = "STAGESELECT";
			changeScene(new MainGameScene(gamedata));
			return;
		}
		
		public function act_gotoranking():void
		{
			changeScene(new ResultScene());
			return;
		}
	}
}

import game.obj.GameObject;
import game.system.Graphic;
import game.system.Database;
import flash.geom.Rectangle;
import flash.geom.Point;
import flash.display.BitmapData;
import flash.geom.Matrix;
import flash.display.BlendMode;
import game.obj.system.Tile;
import flash.geom.ColorTransform;
import flash.display.Sprite;

// --------------------------------//
// コースセレクトの画像
// --------------------------------//
class Course extends GameObject
{
	private var course:int;
	public var selected:Boolean;
	
	public function Course(num:int)
	{
		course = num;
	}
	
	override public function init():void
	{
		registerObject("PRIO_SYSTEM", "DEPTH_MESSAGE");
		
		var temp:BitmapData = Database.getItem("BMP_COURSE");
		var bmp:BitmapData = new BitmapData(temp.width/2, temp.height*3, true, 0);
		bmp.copyPixels(temp, new Rectangle((course-1)*bmp.width, 0, bmp.width, temp.height), new Point(0, 0));
		
		//色違い画像の作成
		var matrix:Matrix = new Matrix();
		matrix.translate(0, temp.height);
		bmp.draw(bmp, matrix, new ColorTransform(1/2, 1/1.8, 1/1.4));
		matrix.translate(0, temp.height);
		bmp.draw(bmp, matrix, new ColorTransform(1, 1, 1, 1, 255, 255, 255));
		
		grp = new Graphic(bmp);
		
		//画像切り出し
		for(var i:int = 0; i < 3; i++)
		{
			var name:String = "";
			switch(i)
			{
			case 0: name = "NORMAL"; break;
			case 1: name = "DARK"; break;
			case 2: name = "FLASH"; break;
			}
			grp.addFrame(name, new Rectangle(0, i*temp.height, bmp.width, temp.height), 0, 0);
		}
		
		if(course == 1)
		{
			x = 32;
			y = 16+8;
		}
		if(course == 2)
		{
			x = 320-216 - 32;
			y = 240-96 - 16;
		}
		
		changeAction("act_main");
		return;
	}
	
	public function act_main():void
	{
		if(count == 0) {
			frame = "FLASH";
			return;
		}
		
		if(selected == true) {
			frame = "NORMAL";
		}
		else {
			frame = "DARK";
		}
	}
	
	public function act_decide():void
	{
		if(count % 6 < 3) {
			frame = "FLASH";
		}
		else {
			frame = "NORMAL";
		}
	}
}

// --------------------------------//
// 背景
// --------------------------------//
class BG extends GameObject
{
	private var sprite:Sprite = new Sprite();
	private var matrix:Matrix = new Matrix();
	
	override public function init():void
	{
		var temp:BitmapData = Database.getItem("BMP_MENUBG");
		
		//箱
		var tile:BitmapData = new BitmapData(48, 48, false, 0);
		tile.copyPixels(temp, new Rectangle(0, 0, 48, 48), new Point(0, 0));
		
		var bg:Tile = createObject(new Tile(tile));
		bg.x = -8;
		bg.vy = -1/4;
		
		//文字ベルト
		var tile:BitmapData = new BitmapData(120, 24, true, 0);
		tile.copyPixels(temp, new Rectangle(48, 0, 120, 24), new Point(0, 0));
		
		var bg:Tile = createObject(new Tile(tile));
		bg.repeat_y = false;
		bg.y = 120-tile.height*1.5+0*tile.height;
		bg.vx = -1/2;
		
		var bg:Tile = createObject(new Tile(tile));
		bg.repeat_y = false;
		bg.y = 120-tile.height*1.5+1*tile.height;
		bg.vx = 1/2;
		
		var bg:Tile = createObject(new Tile(tile));
		bg.repeat_y = false;
		bg.y = 120-tile.height*1.5+2*tile.height;
		bg.vx = -1/2;
		
		//まる
		var tile:BitmapData = new BitmapData(208, 208, true, 0);
		tile.copyPixels(temp, new Rectangle(0, 48, 208, 208), new Point(0, 0));
		
		var bg:Tile = createObject(new Tile(tile));
		bg.vx = 1/3;
		bg.vy = 1/2;
		
		var bg:Tile = createObject(new Tile(tile));
		bg.y = tile.height/2;
		bg.vx = -1/3;
		bg.vy = 1/2;
		
		//セロファン
		sprite.graphics.beginFill(0x202020);
		sprite.graphics.drawRect(160-96, 0, 192, 240);
		
		//このオブジェクトが一番手前
		registerObject("PRIO_BG", "DEPTH_DISTANT");
	}
	
	override public function draw(surface:BitmapData):void
	{
		surface.draw(sprite, matrix, null, BlendMode.OVERLAY);
	}
}
