package game.scene
{
	import game.system.Database;
	import game.system.SceneBase;
	import game.system.GameData;
	import game.obj.system.Message;
	import game.obj.system.Tile;
	import flash.display.BitmapData;
	import flash.geom.Rectangle;
	import flash.geom.Point;
	import game.obj.system.Picture;
	import game.system.PlayData;
	import game.obj.system.FadeIn;
	
	public class ResultScene extends SceneBase
	{
		private var gamedata:GameData;
		private var messages:Object = new Object();
		
		public function ResultScene(gamedata:GameData = null):void
		{
			this.gamedata = gamedata;
		}
		
		override public function init():void
		{
			var bgtile:Tile = createObject(new Tile(Database.getItem("BMP_BOXTILE")));
			bgtile.vy = 1/6;
			
			createObject(new FadeIn(30));
			
			if(gamedata == null) {
				changeAction("act_ranking");
			}
			else {
				changeAction("act_result");
			}
		}
		
		private function removeAllMessages():void
		{
			for each(var mes:Message in messages)
			{
				mes.vanish();
			}
			messages = new Object();
		}
		
		// --------------------------------//
		// ランキング
		// --------------------------------//
		public function act_ranking():void
		{
			if(count == 0)
			{
				messages["ranking"] = new Message("16x8_blue", "RANKING");
				messages["course1"] = new Message("16x8_blue", "COURSE1");
				messages["course2"] = new Message("16x8_blue", "COURSE2");
				
				messages["ranking"].x = 132;
				messages["ranking"].y = 8;
				messages["course1"].x = 60;
				messages["course1"].y = 32;
				messages["course2"].x = 204;
				messages["course2"].y = 32;
				
				createObject(messages["ranking"]);
				createObject(messages["course1"]);
				createObject(messages["course2"]);
				
				messages["c1rank"] = new Message("16x8_blue");
				messages["c2rank"] = new Message("16x8_blue");
				messages["c1name"] = new Message("16x8_blue");
				messages["c2name"] = new Message("16x8_blue");
				messages["c1score"] = new Message("16x8_blue");
				messages["c2score"] = new Message("16x8_blue");
				
				messages["c1rank"].x = 24;
				messages["c1rank"].y = 56;
				messages["c2rank"].x = 168;
				messages["c2rank"].y = 56;
				messages["c1name"].x = 64;
				messages["c1name"].y = 56;
				messages["c2name"].x = 208;
				messages["c2name"].y = 56;
				messages["c1score"].x = 96;
				messages["c1score"].y = 56;
				messages["c2score"].x = 240;
				messages["c2score"].y = 56;
				
				createObject(messages["c1rank"]);
				createObject(messages["c2rank"]);
				createObject(messages["c1name"]);
				createObject(messages["c2name"]);
				createObject(messages["c1score"]);
				createObject(messages["c2score"]);
				
				for(var j:int = 1; j <= 2; j++)
				{
					var ranking:* = PlayData.ranking;
					
					for(var i:int = 1; i <= 10; i++)
					{
						messages["c"+j+"rank"].str += getRankText(i);
						messages["c"+j+"name"].str += ranking.data.name[j][i]+"\n";
						messages["c"+j+"score"].str += MathEx.addZero(ranking.data.score[j][i], 6)+"\n";
					}
				}
			}
			
			if(count > 15) {
				changeAction("act_ranking_wait");
			}
		}
		
		public function act_ranking_wait():void
		{
			if(func_key.isPress("BTN_A") || func_key.isPress("BTN_B"))
			{
				removeAllMessages();
				
				//ネームエントリーの来るとBGMが残ってる
				if(gamedata != null) {
					func_bgm.fadeout();
				}
				
				changeAction_fadeout("act_gotomainmenu");
			}
		}
		
		public function getRankText(rank:int):String
		{
			switch(rank)
			{
			case 1  : return " 1ST\n";
			case 2  : return " 2ND\n";
			case 3  : return " 3RD\n";
			default : return " "+rank+"TH\n";
			case 10 : return "10TH\n";
			}
		}
		
		// --------------------------------//
		// リザルト
		// --------------------------------//
		private var rating:Picture;
		
		public function act_result():void
		{
			if(count == 0)
			{
				messages["result"] = new Message("16x8_blue", "RESULT");
				messages["stage"] = new Message("16x8_orange");
				messages["stagetime"] = new Message("16x8_blue");
				messages["newrecord"] = new Message("16x8_orange");
				
				messages["result"].x = 24;
				messages["result"].y = 8;
				
				messages["stage"].x = 32;
				messages["stage"].y = 32;
				
				messages["stagetime"].x = 96;
				messages["stagetime"].y = 32;
				
				messages["newrecord"].x = 176;
				messages["newrecord"].y = 32;
				
				createObject(messages["result"]);
				createObject(messages["stage"]);
				createObject(messages["stagetime"]);
				createObject(messages["newrecord"]);
				
				for(var i:int = 0; i < gamedata.records.length; i++)
				{
					messages["stage"].str += "STAGE "+(i+1)+"\n";
					messages["stagetime"].str += gamedata.timeToString(gamedata.records[i])+"\n";
					
					if(PlayData.records.isBest(gamedata.course, i+1, gamedata.records[i]))
					{
						messages["newrecord"].str += "NEW RECORD\n";
						PlayData.records.entry(gamedata.course, i+1, gamedata.records[i]);
					}
					else {
						messages["newrecord"].str += "\n";
					}
				}
				PlayData.records.saveData();
				
				messages["txt_time"] = createObject(new Message("16x8_orange", "TIME LEFT"));
				messages["txt_score"] = createObject(new Message("16x8_orange", "SCORE"));
				messages["txt_rating"] = createObject(new Message("16x8_orange", "YOUR RANK IS"));
				messages["time"] = createObject(new Message("16x8_blue", gamedata.timeToString(gamedata.time)));
				messages["score"] = createObject(new Message("16x8_blue", MathEx.addZero(gamedata.score, 6)));
				
				messages["txt_time"].x = 24;
				messages["txt_time"].y = 167;
				messages["time"].x = 104;
				messages["time"].y = 167;
				messages["txt_score"].x = 24;
				messages["txt_score"].y = 185;
				messages["score"].x = 72;
				messages["score"].y = 185;
				messages["txt_rating"].x = 32;
				messages["txt_rating"].y = 216;
				
				var temp:BitmapData = new BitmapData(48, 32, true, 0);
				temp.copyPixels(Database.getItem("BMP_RATING"), new Rectangle(0, getRating()*32, 48, 32), new Point());
				
				rating = createObject(new Picture(temp));
				rating.x = 136;
				rating.y = 204;
			}
			
			if(count > 15) {
				changeAction("act_result_wait");
			}
		}
		
		public function act_result_wait():void
		{
			if(func_key.isPress("BTN_A") || func_key.isPress("BTN_B"))
			{
				removeAllMessages();
				rating.vanish();
				
				if(PlayData.ranking.isRankIn(gamedata.score, gamedata.course) == false)
				{
					changeAction("act_ranking");
				}
				else {
					changeAction("act_entry");
				}
			}
		}
		
		public function getRating():int
		{
			if(gamedata.course == 1)
			{
				if(gamedata.score >= 175000) {return 12;}
				if(gamedata.score >= 150000) {return 11;}
				if(gamedata.score >= 125000) {return 10;}
				if(gamedata.score >= 100000) {return 9;}
				if(gamedata.score >=  80000) {return 8;}
				if(gamedata.score >=  60000) {return 7;}
				if(gamedata.score >=  40000) {return 6;}
				if(gamedata.score >=  30000) {return 5;}
				if(gamedata.score >=  20000) {return 4;}
				if(gamedata.score >=  10000) {return 3;}
				if(gamedata.score >=   6000) {return 2;}
				if(gamedata.score >=   3000) {return 1;}
				return 0;
			}
			else
			{
				if(gamedata.score >= 300000) {return 12;}
				if(gamedata.score >= 260000) {return 11;}
				if(gamedata.score >= 220000) {return 10;}
				if(gamedata.score >= 180000) {return 9;}
				if(gamedata.score >= 150000) {return 8;}
				if(gamedata.score >= 120000) {return 7;}
				if(gamedata.score >= 100000) {return 6;}
				if(gamedata.score >=  80000) {return 5;}
				if(gamedata.score >=  60000) {return 4;}
				if(gamedata.score >=  45000) {return 3;}
				if(gamedata.score >=  30000) {return 2;}
				if(gamedata.score >=  15000) {return 1;}
				return 0;
			}
		}
		
		// --------------------------------//
		// ネームエントリー
		// --------------------------------//
		private const table:String = " ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-'./㍗";
		private var belt:Message;
		private var readpos:int;
		private var writepos:int;
		private var readcursor:Cursor;
		private var writecursor:Cursor;
		
		public function act_entry():void
		{
			var rank:int = PlayData.ranking.isRankIn(gamedata.score, gamedata.course);
			
			if(count == 0)
			{
				func_bgm.play("BGM_ENTRY");
				
				messages["entry"] = new Message("16x8_blue", "ENTRY YOUR NAME");
				messages["rank"] = new Message("16x8_orange", getRankText(rank));
				messages["name"] = new Message("16x8_blue", "   ");
				
				messages["entry"].x = 160-messages["entry"].str.length*4;
				messages["entry"].y = 88;
				messages["rank"].x = 120;
				messages["rank"].y = 112;
				messages["name"].x = 168;
				messages["name"].y = 112;
				
				createObject(messages["entry"]);
				createObject(messages["rank"]);
				createObject(messages["name"]);
				
				belt = new Message("16x8_blue", table);
				readcursor = new Cursor();
				writecursor = new Cursor();
				
				belt.x = 160-4;
				belt.y = 136;
				readcursor.x = 160-4;
				readcursor.y = 136;
				writecursor.x = 168;
				writecursor.y = 112;
				
				createObject(belt);
				createObject(readcursor);
				createObject(writecursor);
			}
			
			if(func_key.isPress("LEFT", true) && readpos > 0)
			{
				playSE("SE_SELECT");
				readpos--;
			}
			else if(func_key.isPress("RIGHT", true) && readpos < belt.str.length-1)
			{
				playSE("SE_SELECT");
				readpos++;
			}
			
			belt.x = 160-4 - readpos*8;
			
			var char:String = table.charAt(readpos);
			var str:String = messages["name"].str;
			
			if(func_key.isPress("BTN_A"))
			{
				playSE("SE_ENTER");
				
				if(char == "㍗")
				{
					PlayData.ranking.insert(rank, messages["name"].str, gamedata.score, gamedata.course);
					changeAction("act_entry_decide");
					return;
				}
				
				messages["name"].str = str.slice(0,writepos)+char+str.slice(writepos+1);
				
				writepos++;
				if(writepos >= 3) {
					writepos = 3-1;
					readpos = table.length - 1;
				}
			}
			else if(func_key.isPress("BTN_B"))
			{
				playSE("SE_CANCEL");
				messages["name"].str = str.slice(0,writepos)+" "+str.slice(writepos+1);
				
				writepos--;
				if(writepos < 0) {
					writepos = 0;
					messages["name"].str = "   ";
				}
			}
			
			writecursor.x = 168 + writepos*8;
		}
		
		public function act_entry_decide():void
		{
			if(count == 0)
			{
				belt.str = "㍗";
				belt.x = 160-4;
				
				readcursor.vanish();
				writecursor.vanish();
			}
			
			if(count <= 12)
			{
				if(count % 6 < 3) {
					belt.visible = false;
				}
				else {
					belt.visible = true;
				}
			}
			
			if(count == 15)
			{
				removeAllMessages();
				belt.vanish();
			}
			
			if(count == 25)
			{
				PlayData.ranking.saveData();
				changeAction("act_ranking");
			}
		}
		
		// --------------------------------//
		// シーン変更
		// --------------------------------//
		public function act_gotomainmenu():void
		{
			changeScene(new MenuScene());
		}
	}
}

import game.obj.GameObject;
import flash.display.BitmapData;
import flash.geom.Point;
import flash.geom.Matrix;

class Cursor extends GameObject
{
	private var rect:BitmapData = new BitmapData(8, 16, false, 0xFFFFFFFF);
	private var matrix:Matrix = new Matrix();
	
	override public function init():void
	{
		registerObject("PRIO_SYSTEM", "DEPTH_SYSTEM");
	}
	
	override public function draw(surface:BitmapData):void
	{
		if(count % 9 < 3) {
			surface.copyPixels(rect, rect.rect, new Point(x, y));
		}
		else if(count % 9 < 6)
		{
			matrix.identity();
			matrix.translate(x, y);
			surface.draw(rect, matrix, null, "invert");
		}
	}
}
