package game.system
{
	import common.Screen;
	import common.List;
	import game.system.ObjBase;
	
	public class ObjectManager
	{
		private var screen:Screen;
		
		private static var priolist:Object = new Array();
		private static var depthlist:Array = new Array();
		
		private var objlist:Object = new Object();
		private var grplist:Object = new Object();
		private var deletelist:Array = new Array();
		
		private var lockflag:Boolean = false;
		
		// --------------------------------//
		// 初期化
		// --------------------------------//
		public function ObjectManager(scr:Screen)
		{
			screen = scr;
			
			for each(var prio:String in priolist) {
				objlist[prio] = new List();
			}
			
			for each(var depth:String in depthlist) {
				grplist[depth] = new List();
			}
			
			return;
		}
		
		// --------------------------------//
		// プライオリティ追加
		// --------------------------------//
		public static function addPriority(prio:String):void
		{
			priolist.push(prio);
			return;
		}
		
		// --------------------------------//
		// 深度追加
		// --------------------------------//
		public static function addDepth(depth:String):void
		{
			depthlist.push(depth);
			return;
		}
		
		// --------------------------------//
		// 登録
		// --------------------------------//
		public function register(obj:ObjBase, prio:String, depth:String):void
		{
			objlist[prio].push(obj);
			grplist[depth].push(obj);
			
			obj.priority = prio;
			obj.depth = depth;
			
			obj.visible = true;
			obj.deleteflag = false;
			return;
		}
		
		public function changeDepth(obj:ObjBase, newdepth:String):void
		{
			grplist[obj.depth].erase(obj);
			grplist[newdepth].push(obj);
			
			obj.depth = newdepth;
			return;
		}
		
		// --------------------------------//
		// 削除
		// --------------------------------//
		public function removeMovieClip(obj:ObjBase):void
		{
			//update()でまとめて消される
			obj.deleteflag = true;
			deletelist.push(obj);
			return;
		}
		
		// --------------------------------//
		// 指定プライオリティを全削除
		// --------------------------------//
		public function removePriority(prio:String):void
		{
			var objlist:List = this.searchPriority(prio);
			for each(var obj:* in objlist) {
				this.removeMovieClip(obj);
			}
			
			return;
		}
		
		// --------------------------------//
		// 更新
		// --------------------------------//
		public function update():void
		{
			for each(var obj:ObjBase in deletelist)
			{
				obj.erase();	//デストラクタ
				
				grplist[obj.depth].erase(obj);
				objlist[obj.priority].erase(obj);
			}
			
			deletelist = new Array();
			return;
		}
		
		// --------------------------------//
		// 実行
		// --------------------------------//
		public function run():void
		{
			if(lockflag == false) {
				for(var i:uint = 0; i < priolist.length; i++) {
					for each(var obj:ObjBase in objlist[priolist[i]])
					{
						obj.main();
					}
				}
			}
			else
			{
				for each(var obj:ObjBase in objlist["PRIO_EFFECT"]) {obj.main();}
				for each(var obj:ObjBase in objlist["PRIO_SYSTEM"]) {obj.main();}
			}
			
			update();
			
			return;
		}
		
		// --------------------------------//
		// 描画
		// --------------------------------//
		public function draw():void
		{
			for each(var depth:* in depthlist)
			{
				for each(var obj:* in grplist[depth])
				{
					if(obj.visible == false) {continue;}
					obj.draw(screen.buffer);
				}
			}
			
			return;
		}
		
		// --------------------------------//
		// 全削除
		// --------------------------------//
		public function removeAll():void
		{
			for(var i:uint = 0; i < length; i++)
			{
				for(var j:uint = 0; j < objlist[i].length; j++)
				{
					this.removeMovieClip(objlist[i][j]);
				}
			}
			
			return;
		}
		
		// --------------------------------//
		// ロック
		// --------------------------------//
		public function lock():void
		{
			lockflag = true;
			return;
		}
		
		public function unlock():void
		{
			lockflag = false;
			return;
		}
		
		// --------------------------------//
		// 指定プライオリティのリストを返す
		// --------------------------------//
		public function searchPriority(prio:String):List
		{
			return objlist[prio];
		}
	}
}