package game.system
{
	import common.LocalDataManager;
	
	public class Ranking
	{
		private const RANKING_MAX:uint = 10;
		private const COURSE_MAX:uint = 2;
		private var func_ldm:LocalDataManager = new LocalDataManager("ranking");
		public var data:Array = new Array();
		
		public function Ranking()
		{
			data.score = new Array();
			data.name  = new Array();
			
			for(var j:int = 1; j <= COURSE_MAX; j++)
			{
				data.score[j] = new Array();
				data.name[j]  = new Array();
				
				for(var i:int = 1; i <= RANKING_MAX; i++)
				{
					func_ldm.setDefault("score"+j+"-"+i, 10000 - 1000*(i-1));
					func_ldm.setDefault("name"+j+"-"+i, "90K");
				}
			}
		}
		
		// --------------------------------//
		// 初期化
		// --------------------------------//
		public function initialize():void
		{
			func_ldm.initialize();
			
			for(var j:int = 1; j <= COURSE_MAX; j++)
			{
				for(var i:int = 1; i <= RANKING_MAX; i++)
				{
					data.score[j][i] = func_ldm.getDefault("score"+j+"-"+i);
					data.name[j][i]  = func_ldm.getDefault("name"+j+"-"+i);
				}
			}
			
			return;
		}
		
		// --------------------------------//
		// 保存
		// --------------------------------//
		public function saveData():void
		{
			for(var j:int = 1; j <= COURSE_MAX; j++)
			{
				for(var i:int = 1; i <= RANKING_MAX; i++)
				{
					func_ldm.saveData("score"+j+"-"+i, data.score[j][i]);
					func_ldm.saveData("name"+j+"-"+i,  data.name[j][i]);
				}
			}
			
			return;
		}
		
		// --------------------------------//
		// ローカルデータから読み込み
		// --------------------------------//
		public function loadData():void
		{
			for(var j:int = 1; j <= COURSE_MAX; j++)
			{
				for(var i:int = 1; i <= RANKING_MAX; i++)
				{
					data.score[j][i] = func_ldm.loadData("score"+j+"-"+i);
					data.name[j][i]  = func_ldm.loadData("name"+j+"-"+i);
				}
			}
			
			return;
		}
		
		// --------------------------------//
		// 判定
		// --------------------------------//
		public function isRankIn(score:Number, course:uint):*
		{
			for(var i:int = 1; i <= RANKING_MAX; i++)
			{
				if(data.score[course][i] < score)
				{
					return i;
				}
			}
			
			return false;
		}
		
		// --------------------------------//
		// 順位を下にずらす処理
		// --------------------------------//
		public function insert(rank:uint, name:String, score:uint, course:uint):void
		{
			for(var i:int = RANKING_MAX; i >= rank; i--)
			{
				if(i < RANKING_MAX)
				{
					data.score[course][i+1] = data.score[course][i];
					data.name[course][i+1]  = data.name[course][i];
				}
				
				data.name[course][i] = name;
				data.score[course][i] = score;
			}
			
			return;
		}
		
		// --------------------------------//
		// チェック
		// --------------------------------//
		public function isEmpty():Boolean
		{
			return func_ldm.isEmpty();
		}
		
		public function isError():Boolean
		{
			return func_ldm.isError();
		}
	}
}
